import useDrupalJsonApi from 'hooks/drupal_jsonapi';
import LabelBlock from 'molecules/formblocks/LabelBlock';
import { useState } from 'react';
import { Form } from 'react-bootstrap';
import { useParams } from 'react-router-dom';

/**
 * Get the record data from the URL.
 * @param cloudContext
 * @param entityId
 * @param formData
 * @param url
 * @returns recordList, isLoading
 */
const useRecordData = (
  cloudContext: string,
  entityId: string,
  formData: Record<string, any>,
  url: string,
) => {

  const { getJsonData } = useDrupalJsonApi();
  const [recordList, setRecordList] = useState<{ value: number, label: string, group?: string }[]>([]);
  const [isLoading, setIsLoading] = useState<boolean>(true);

  // Replace the part of the URL that corresponds to the keyword.
  let replacedUrl = url
    .replaceAll('{cloud_context}', cloudContext)
    .replaceAll('{entity_id}', entityId);
  const result = replacedUrl.match(/\{[^{}]+\}/g);
  if (result !== null) {
    for (const pattern of result) {
      const key = pattern.substring(1, pattern.length - 1);
      replacedUrl = replacedUrl.replaceAll(pattern, `${formData[key]}`);
    }
  }
  if (replacedUrl.endsWith('/')) {
    replacedUrl = replacedUrl.substring(0, replacedUrl.length - 1);
  }

  if (isLoading) {
    getJsonData<{ value: number, label: string, group?: string }[]>(replacedUrl, []).then((jsonData) => {
      setRecordList(jsonData);
      setIsLoading(false);
    })
  }

  return {
    recordList,
    isLoading,
  }

}

/**
 * Block of select view implementation.
 */
const UrlMultiCheckBoxImpl = ({
  label, value, setValue, recordList, required, readOnly, id, className
}: {
  label: string,
  value: string[],
  setValue: (s: string[]) => void,
  recordList: { value: number, label: string, group?: string }[],
  required: boolean,
  readOnly: boolean,
  id?: string,
  className?: string,
}) => {

  if (readOnly) {
    const filtered = recordList.filter((r) => value.includes(r.value.toString()));
    return <LabelBlock
      name={label}
      value={
        filtered.length >= 1
          ? filtered.map((r) => r.label).join(', ')
          : value.map((v) => v).join(', ')
      }
      id={id}
      className={className} />;
  }

  const divClassName = 'form-item js-form-item form-type-item' + ((className) ? ' ' + className : '');

  return <>
    <div className={divClassName}>
      <Form.Label htmlFor={id} className={'control-label' + (required ? ' form-required' : '')}>{label}</Form.Label>
    </div>
    <div className="form-group js-form-wrapper form-wrapper">
      <fieldset className={'fieldgroup form-composite form-item js-form-item form-wrapper' + (required ? ' required' : '')}>
        <legend className="d-none">
          <span className={'fieldset-legend' + (required ? ' form-required' : '')}>{label}</span>
        </legend>
        <div className="fieldset-wrapper">
          <div className="form-checkboxes">
            {
              recordList.map((d, index) => {
                const idText = `${id !== undefined ? id : 'multi-select'}-${index}`;
                return <div key={d.value} className="form-item form-type-checkbox">
                  <input type="checkbox" className="form-checkbox form-check-input" id={idText}
                    checked={value.includes(`${d.value}`)}
                    onClick={() => {
                      if (value.includes(`${d.value}`)) {
                        setValue(value.filter(v => v !== `${d.value}`));
                      } else {
                        setValue([...value, `${d.value}`]);
                      }
                    }} />
                  <Form.Label className="control-label option"
                    for={idText}>
                    <span className="views-field views-field-name">
                      <span className="field-content">
                        {d.label}
                      </span>
                    </span>
                  </Form.Label>
                </div>;
              })
            }
          </div>
        </div>
      </fieldset>
    </div>
  </>;

}

/**
 * Block of select view.
 *
 * @param
 */
const UrlMultiCheckBox = ({ label, value, setValue, formData, url, cloudContext, required, readOnly, id, className }: {
  label: string,
  value: string[],
  setValue: (s: string[]) => void,
  formData: Record<string, any>,
  url: string,
  cloudContext: string,
  required: boolean,
  readOnly?: boolean,
  id?: string,
  className?: string
}) => {

  const params: {
    entityId: string,
  } = useParams();

  const { recordList, isLoading } = useRecordData(cloudContext, params.entityId, formData, url);

  return isLoading
    ? <></>
    : <UrlMultiCheckBoxImpl
      label={label}
      value={value}
      setValue={setValue}
      recordList={recordList}
      required={required}
      readOnly={readOnly ?? false}
      id={id}
      className={className} />

}

export default UrlMultiCheckBox;
