<?php

namespace Drupal\cloud\Controller;

use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityDeleteFormTrait;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller responsible for "update" URLs.
 *
 * This class is mainly responsible for updating the entities from URLs.
 */
class ApiController extends ControllerBase implements ApiControllerInterface {

  use CloudContentEntityTrait;
  use EntityDeleteFormTrait;

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  private $cloudConfigPluginManager;

  /**
   * The entity used by this form.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface|\Drupal\Core\Entity\RevisionLogInterface
   */
  private $entity;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * ApiController constructor.
   *
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    EntityTypeManagerInterface $entity_type_manager,
    ModuleHandlerInterface $module_handler
  ) {
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return ApiController
   *   return created object.
   */
  public static function create(ContainerInterface $container): ApiController {
    return new static(
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('entity_type.manager'),
      $container->get('module_handler')
    );
  }

  /**
   * Gets the entity of this form.
   *
   * Provided by \Drupal\Core\Entity\EntityForm.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The entity.
   */
  public function getEntity(): EntityInterface {
    return $this->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function operateEntity(Request $request, string $cloud_service_provider, string $cloud_context, string $entity_type_id, string $entity_id, string $command): JsonResponse {
    switch ($command) {
      case 'delete':
        try {
          /** @var \Drupal\cloud\Entity\CloudLaunchTemplateInterface $entity */
          $entity = $this->entityTypeManager
            ->getStorage($entity_type_id)
            ->load($entity_id);
          if (empty($entity)) {
            return new JsonResponse([
              'result' => 'NG',
              'reason' => 'The entity has already been deleted.',
            ], 404);
          }

          $this->entity = $entity;
          $entity->delete();

          $this->messenger->addStatus($this->getDeletionMessage());
          $this->logDeletionMessage();

          // Clear block and menu cache.
          $this->clearCacheValues($this->entity->getCacheTags());
          return new JsonResponse([
            'result' => 'OK',
          ], 200);
        }
        catch (\Exception $e) {
          $this->handleException($e);
          return new JsonResponse([
            'result' => 'NG',
            'reason' => 'Internal Server Error',
          ], 500);
        }

      default:
        return new JsonResponse([
          'result' => 'NG',
          'reason' => 'Not Found',
        ], 404);
    }

  }

  /**
   * {@inheritdoc}
   */
  public function getImageOptions($cloud_service_provider, $cloud_context): JsonResponse {
    /** @var \Drupal\aws_cloud\Entity\Ec2\ImageInterface[] $images */
    $images = $this->entityTypeManager
      ->getStorage("{$cloud_service_provider}_image")
      ->loadByProperties([
        'cloud_context' => $cloud_context,
      ]);

    $options = [];
    foreach ($images ?: [] as $image) {
      $options[$image->getImageId()] = sprintf('%s (%s)',
        $image->getName(),
        $image->getImageId()
      );
    }

    natcasesort($options);

    $output = [];
    foreach ($options ?: [] as $value => $label) {
      $output[] = [
        'value' => $value,
        'label' => $label,
      ];
    }
    return new JsonResponse($output, 200);
  }

  /**
   * {@inheritdoc}
   */
  public function getVisibleManageMenuLinks(): JsonResponse {
    $currentUser = $this->currentUser();
    $visibleMenuItemList = [];

    if (!$currentUser->isAuthenticated()) {
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'Invalid request.',
      ], 400);
    }

    if ($currentUser->hasPermission('view cloud service provider admin list')) {
      $visibleMenuItemList[] = [
        'label' => 'Administration',
        'url' => Url::fromRoute('entity.cloud_config.collection')->toString(),
      ];
    }

    if ($this->moduleHandler->moduleExists('roleassign') && $currentUser->hasPermission('administer users')) {
      $visibleMenuItemList[] = [
        'label' => 'Users',
        'url' => Url::fromRoute('entity.user.collection')->toString(),
      ];
    }

    if ($this->moduleHandler->moduleExists('module_permissions') && $currentUser->hasPermission('administer managed modules permissions')) {
      $visibleMenuItemList[] = [
        'label' => 'Permissions',
        'url' => Url::fromRoute('user.admin_permissions')->toString(),
      ];
    }

    return new JsonResponse($visibleMenuItemList);
  }

}
